"""Support for LK3."""
import asyncio
import logging
from datetime import timedelta
from typing import Any, Dict, Optional, Union

from homeassistant.components.sensor import DOMAIN as SENSOR_DOMAIN
from homeassistant.components.switch import DOMAIN as SWITCH_DOMAIN
from homeassistant.components.binary_sensor import DOMAIN as BINARY_SENSOR_DOMAIN
from homeassistant.config_entries import ConfigEntry
from homeassistant.const import (ATTR_NAME, CONF_HOST, CONF_PASSWORD,
                                 CONF_USERNAME)
from homeassistant.core import HomeAssistant, callback
from homeassistant.exceptions import ConfigEntryNotReady
from homeassistant.helpers.aiohttp_client import async_get_clientsession
from homeassistant.helpers.dispatcher import (async_dispatcher_connect,
                                              async_dispatcher_send)
from homeassistant.helpers.entity import Entity
from homeassistant.helpers.event import async_track_time_interval
from homeassistant.helpers.typing import ConfigType
from homeassistant.util import dt as dt_util

from .lk3api import LK3Error, LK3Reader
from .const import (ATTR_IDENTIFIERS, ATTR_MANUFACTURER, ATTR_MODEL,
                    ATTR_SOFTWARE_VERSION, DATA_LK3_CLIENT, DATA_LK3_TIMER,
                    DATA_LK3_UPDATED, DOMAIN, UPDATE_INTERVAL)

LK3_COMPONENTS = (SENSOR_DOMAIN, SWITCH_DOMAIN, BINARY_SENSOR_DOMAIN)

_LOGGER = logging.getLogger(__name__)


async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
    """Set up the LK3 components."""
    return True


async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
    """Set up LK3 from a config entry."""
    # Create LK3Reader instance for this entry
    session = async_get_clientsession(hass)
    lk3reader = LK3Reader(
        entry.data[CONF_HOST],
        entry.data[CONF_USERNAME],
        entry.data[CONF_PASSWORD],
        session=session,
    )

    # Ensure it's possible to connect to it
    try:
        await lk3reader.update()
    except LK3Error as exc:
        raise ConfigEntryNotReady from exc

    hass.data.setdefault(DOMAIN, {})
    hass.data[DOMAIN][entry.entry_id] = {DATA_LK3_CLIENT: lk3reader}

    # Set up platforms for this device/entry.
    for component in LK3_COMPONENTS:
        hass.async_create_task(
            hass.config_entries.async_forward_entry_setup(entry, component)
        )

    async def interval_update(now: dt_util.dt.datetime = None) -> None:
        """Poll LK3 device function, dispatches event after update."""
        try:
            await lk3reader.update()
        except LK3Error:
            _LOGGER.debug('An error occuren while updating LK3', exc_info=True)
        async_dispatcher_send(hass, DATA_LK3_UPDATED, entry.entry_id)

    # Schedule update interval
    hass.data[DOMAIN][entry.entry_id][DATA_LK3_TIMER] = async_track_time_interval(
        hass, interval_update, timedelta(seconds=entry.data[UPDATE_INTERVAL])
    )

    return True


async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
    """Unload LK3 config entry."""

    # Cancel update timer for this entry/device.
    cancel_timer = hass.data[DOMAIN][entry.entry_id][DATA_LK3_TIMER]
    cancel_timer()

    # Unload entities for this entry/device.
    await asyncio.gather(
        *(
            hass.config_entries.async_forward_entry_unload(entry, component)
            for component in LK3_COMPONENTS
        )
    )

    # Cleanup
    del hass.data[DOMAIN][entry.entry_id]
    if not hass.data[DOMAIN]:
        del hass.data[DOMAIN]

    return True


class LK3Entity(Entity):
    """Defines a base LK3 entity."""
    def __init__(self, entry_id: str, reader: LK3Reader, name: str, icon: str) -> None:
        """Initialize the LK3 entity."""
        self._attributes: Dict[str, Union[str, int, float]] = {}
        self._available = True
        self._entry_id = entry_id
        self._icon = icon
        self._name = name
        self._unsub_dispatcher = None
        self.reader = reader

    @property
    def name(self) -> str:
        """Return the name of the entity."""
        return self._name

    @property
    def icon(self) -> str:
        """Return the mdi icon of the entity."""
        return self._icon

    @property
    def available(self) -> bool:
        """Return True if entity is available."""
        return self._available

    @property
    def should_poll(self) -> bool:
        """Return the polling requirement of the entity."""
        return False

    @property
    def device_state_attributes(self) -> Optional[Dict[str, Any]]:
        """Return the state attributes of the entity."""
        return self._attributes

    async def async_added_to_hass(self) -> None:
        """Connect to dispatcher listening for entity data notifications."""
        self._unsub_dispatcher = async_dispatcher_connect(
            self.hass, DATA_LK3_UPDATED, self._schedule_immediate_update
        )

    async def async_will_remove_from_hass(self) -> None:
        """Disconnect from update signal."""
        self._unsub_dispatcher()

    @callback
    def _schedule_immediate_update(self, entry_id: str) -> None:
        """Schedule an immediate update of the entity."""
        if entry_id == self._entry_id:
            self.async_schedule_update_ha_state(True)

    async def async_update(self) -> None:
        """Update LK3 entity."""
        if self.reader.device is None:
            self._available = False
            return

        self._available = True
        await self._lk3_update()

    async def _lk3_update(self) -> None:
        """Update LK3 entity."""
        raise NotImplementedError()


class LK3DeviceEntity(LK3Entity):
    """Defines a LK3 device entity."""

    @property
    def device_info(self) -> Dict[str, Any]:
        """Return device information about this LK3 device."""
        return {
            ATTR_IDENTIFIERS: {(DOMAIN, self.reader.device.info.mac_address)},
            ATTR_NAME: self.reader.device.info.name,
            ATTR_MANUFACTURER: self.reader.device.info.brand,
            ATTR_MODEL: self.reader.device.info.model,
            ATTR_SOFTWARE_VERSION: self.reader.device.info.sw_version,
        }
