"""Models for LK3."""
import attr


@attr.s(auto_attribs=True, frozen=True)
class Info:
    """Object holding information from LK3."""
    mac_address: str
    sw_version: str
    model: str
    name: str
    brand: str

    @staticmethod
    def from_dict(data: dict):
        """Return Info object from LK3 API response."""
        return Info(
            mac_address=data.get('mac', ''),
            sw_version=data.get('sw', 'Unknown'),
            model=data.get('hw', 'Unknown'),
            name='LK3',
            brand='Tinycontrol',
        )


@attr.s(auto_attribs=True, frozen=True)
class State:
    """Object holding information from LK3 sensors."""
    out0: bool
    out1: bool
    out2: bool
    out3: bool
    out4: bool
    out5: bool
    pwm0: bool
    pwm1: bool
    pwm2: bool
    pwm3: bool
    vin: float
    tem: float
    ds1: float
    ds2: float
    ds3: float
    ds4: float
    ds5: float
    ds6: float
    ds7: float
    ds8: float
    t1: float
    h1: float
    p1: float
    pm1: float
    pm2: float
    pm4: float
    pm10: float
    co2: int
    diff1: float
    diff2: float
    diff3: float
    inpa1: float
    inpa2: float
    inpa3: float
    inpa4: float
    inpa5: float
    inpa6: float
    inpd1: bool
    inpd2: bool
    inpd3: bool
    inpd4: bool
    power1: float
    power2: float
    power3: float
    power4: float
    energy1: float
    energy2: float
    energy3: float
    energy4: float
    duralux0: float
    duralux1: float
    duralux2: float
    duralux3: float
    duralux4: float
    duralux5: float
    duralux6: float
    duralux7: float
    duralux8: float
    sdm1: float
    sdm2: float
    sdm3: float
    sdm4: float
    sdm5: float
    sdm6: float
    sdm7: float
    sdm8: float
    sdm9: float
    sdm10: float
    sdm11: float
    sdm12: float
    sdm13: float
    sdm14: float
    sdm15: float
    sdm16: float
    sdm17: float
    sdm18: float
    sdm19: float
    sdm20: float
    sdm21: float
    sdm22: float
    sdm23: float
    sdm24: float
    sdm25: float
    sdm26: float
    sdm27: float
    sdm28: float
    sdm29: float

    def __getitem__(self, key):
        """Make object subscriptable (self[key])."""
        return self.__dict__[key]

    @staticmethod
    def decimal_to_bool_list(data, length):
        """Convert decimal string to list of bool values.

        Eg. decimal_to_bool_list('3', 4) -> [True, True, False, False]
        """
        return [
            bool(int(item))
            for item in reversed(('{:0' + str(length) + 'b}').format(int(data)))
        ]

    @staticmethod
    def parse_diff(value, source):
        """Parse value of diff according to source sensor."""
        if source < 6:
            return value/100
        elif source < 14 or source >= 22:
            return value/10
        else:
            return value/1000

    @staticmethod
    def parse_sdm(data):
        """Parse sdms from data."""
        sw_version = data.get('sw')
        hw_version = data.get('hw')
        if hw_version is not None and sw_version is not None:
            if hw_version == '3.0' or (hw_version != '3.0' and sw_version <= '1.31'):
                # Dict method for sdm1-sdm14, rest is unknown
                return [int(data.get(f'sdm{i}', 0))/100 for i in range(1, 30)]
            else:
                # List method for sdm1-sdm29 for hw 3.5+ sw 1.32+
                modbus_sensor = int(data.get('modbusSensor', 0))
                sdm_list = [int(item) for item in data.get('sdm')]
                if modbus_sensor == 5:
                    return [
                        *sdm_list[0:6], # 0-5
                        sdm_list[6]/10,
                        sdm_list[7]/100,
                        sdm_list[8]/10,
                        sdm_list[9]/100,
                        sdm_list[10]/100,
                        sdm_list[11]/100,
                        sdm_list[12]/100,
                        sdm_list[13]/100,
                        sdm_list[14]/100,
                        sdm_list[15]/10,
                        sdm_list[16]/100,
                        sdm_list[17]/10,
                        sdm_list[18]/100,
                        sdm_list[19]/10,
                        sdm_list[20]/100,
                        *sdm_list[21:25], # 21-24
                        sdm_list[25]/100,
                        *sdm_list[26:29], # 26-28
                    ]
                elif modbus_sensor == 4:
                    return [
                        *[item/100 for item in sdm_list[0:9]], # 0-8
                        *sdm_list[9:13], # 9-12
                        *[item/100 for item in sdm_list[13:29]], # 13-28
                    ]
                else:
                    return [item/100 for item in sdm_list]
        return [0]*29

    @staticmethod
    def from_dict(data):
        """Return State object from LK3 API response."""
        pwm = State.decimal_to_bool_list(data.get('pwm', 0), 4)
        inpd = State.decimal_to_bool_list(data.get('ind', 0), 4)
        diffsel = list(map(int, data.get('diffsel', '').split('*')))
        sdm = State.parse_sdm(data)
        return State(
            out0=bool(int(data.get('out0', 0))),
            out1=bool(int(data.get('out1', 0))),
            out2=bool(int(data.get('out2', 0))),
            out3=bool(int(data.get('out3', 0))),
            out4=bool(int(data.get('out4', 0))),
            out5=bool(int(data.get('out5', 0))),
            pwm0=pwm[0],
            pwm1=pwm[1],
            pwm2=pwm[2],
            pwm3=pwm[3],
            vin=int(data.get('vin', 0))/100,
            tem=int(data.get('tem', 0))/100,
            ds1=int(data.get('ds1', 0))/10,
            ds2=int(data.get('ds2', 0))/10,
            ds3=int(data.get('ds3', 0))/10,
            ds4=int(data.get('ds4', 0))/10,
            ds5=int(data.get('ds5', 0))/10,
            ds6=int(data.get('ds6', 0))/10,
            ds7=int(data.get('ds7', 0))/10,
            ds8=int(data.get('ds8', 0))/10,
            t1=int(data.get('dthTemp', 0))/10,
            h1=int(data.get('dthHum', 0))/10,
            p1=int(data.get('bm280p', 0))/10,
            pm1=int(data.get('pm1', 0))/10,
            pm2=int(data.get('pm2', 0))/10,
            pm4=int(data.get('pm4', 0))/10,
            pm10=int(data.get('pm10', 0))/10,
            co2=int(data.get('co2', 0)),
            diff1=State.parse_diff(int(data.get('diff1', 0)), diffsel[0]),
            diff2=State.parse_diff(int(data.get('diff2', 0)), diffsel[1]),
            diff3=State.parse_diff(int(data.get('diff3', 0)), diffsel[2]),
            inpa1=int(data.get('inpp1', 0))/100,
            inpa2=int(data.get('inpp2', 0))/100,
            inpa3=int(data.get('inpp3', 0))/100,
            inpa4=int(data.get('inpp4', 0))/100,
            inpa5=int(data.get('inpp5', 0))/100,
            inpa6=int(data.get('inpp6', 0))/100,
            inpd1=inpd[0],
            inpd2=inpd[1],
            inpd3=inpd[2],
            inpd4=inpd[3],
            power1=int(data.get('power1', 0))/1000,
            power2=int(data.get('power2', 0))/1000,
            power3=int(data.get('power3', 0))/1000,
            power4=int(data.get('power4', 0))/1000,
            energy1=int(data.get('energy1', 0))/1000,
            energy2=int(data.get('energy2', 0))/1000,
            energy3=int(data.get('energy3', 0))/1000,
            energy4=int(data.get('energy4', 0))/1000,
            duralux0=int(data.get('d0', 0))/10,
            duralux1=int(data.get('d1', 0))/10,
            duralux2=int(data.get('d2', 0))/10,
            duralux3=int(data.get('d3', 0))/10,
            duralux4=int(data.get('d4', 0))/100,
            duralux5=int(data.get('d5', 0)),
            duralux6=int(data.get('d6', 0)),
            duralux7=int(data.get('d7', 0))/10,
            duralux8=int(data.get('d8', 0)),
            **{f'sdm{index+1}': value for index, value in enumerate(sdm)}, # sdm1-sdm29
        )


@attr.s(auto_attribs=True, frozen=True)
class Device:
    """Object holding all information of LK3."""
    info: Info
    state: State

    @staticmethod
    def from_dict(data):
        """Return Device object from LK3 API response."""
        return Device(
            info=Info.from_dict(data),
            state=State.from_dict(data),
        )
