"""Support for LK3 sensors."""
import logging
from typing import Callable, List, Optional, Union

from homeassistant.config_entries import ConfigEntry
from homeassistant.const import (DEVICE_CLASS_HUMIDITY, DEVICE_CLASS_POWER,
                                 DEVICE_CLASS_PRESSURE,
                                 DEVICE_CLASS_TEMPERATURE,
                                 ENERGY_KILO_WATT_HOUR, POWER_WATT,
                                 PRESSURE_HPA, TEMP_CELSIUS)
from homeassistant.helpers.entity import Entity
from homeassistant.helpers.typing import HomeAssistantType

from . import LK3DeviceEntity
from .const import DATA_LK3_CLIENT, DOMAIN
from .lk3api import LK3Reader

_LOGGER = logging.getLogger(__name__)

SENSOR_LIST = [
    {'name': 'VCC', 'key': 'vin', 'device_class': None, 'unit': 'V'},
    {'name': 'BoardTemp', 'key': 'tem', 'device_class': DEVICE_CLASS_TEMPERATURE, 'unit': TEMP_CELSIUS},
    {'name': 'DS1', 'key': 'ds1', 'device_class': DEVICE_CLASS_TEMPERATURE, 'unit': TEMP_CELSIUS},
    {'name': 'DS2', 'key': 'ds2', 'device_class': DEVICE_CLASS_TEMPERATURE, 'unit': TEMP_CELSIUS},
    {'name': 'DS3', 'key': 'ds3', 'device_class': DEVICE_CLASS_TEMPERATURE, 'unit': TEMP_CELSIUS},
    {'name': 'DS4', 'key': 'ds4', 'device_class': DEVICE_CLASS_TEMPERATURE, 'unit': TEMP_CELSIUS},
    {'name': 'DS5', 'key': 'ds5', 'device_class': DEVICE_CLASS_TEMPERATURE, 'unit': TEMP_CELSIUS},
    {'name': 'DS6', 'key': 'ds6', 'device_class': DEVICE_CLASS_TEMPERATURE, 'unit': TEMP_CELSIUS},
    {'name': 'DS7', 'key': 'ds7', 'device_class': DEVICE_CLASS_TEMPERATURE, 'unit': TEMP_CELSIUS},
    {'name': 'DS8', 'key': 'ds8', 'device_class': DEVICE_CLASS_TEMPERATURE, 'unit': TEMP_CELSIUS},
    {'name': 'T1', 'key': 't1', 'device_class': DEVICE_CLASS_TEMPERATURE, 'unit': TEMP_CELSIUS},
    {'name': 'H1', 'key': 'h1', 'device_class': DEVICE_CLASS_HUMIDITY, 'unit': '%'},
    {'name': 'P1', 'key': 'p1', 'device_class': DEVICE_CLASS_PRESSURE, 'unit': PRESSURE_HPA},
    {'name': 'PM1.0', 'key': 'pm1', 'device_class': None, 'unit': 'μm/m3'},
    {'name': 'PM2.5', 'key': 'pm2', 'device_class': None, 'unit': 'μm/m3'},
    {'name': 'PM4.0', 'key': 'pm4', 'device_class': None, 'unit': 'μm/m3'},
    {'name': 'PM10.0', 'key': 'pm10', 'device_class': None, 'unit': 'μm/m3'},
    {'name': 'CO2', 'key': 'co2', 'device_class': None, 'unit': 'ppm'},
    {'name': 'DIFF1', 'key': 'diff1', 'device_class': None, 'unit': None},
    {'name': 'DIFF2', 'key': 'diff2', 'device_class': None, 'unit': None},
    {'name': 'DIFF3', 'key': 'diff3', 'device_class': None, 'unit': None},
    {'name': 'INPA1', 'key': 'inpa1', 'device_class': None, 'unit': 'V'},
    {'name': 'INPA2', 'key': 'inpa2', 'device_class': None, 'unit': 'V'},
    {'name': 'INPA3', 'key': 'inpa3', 'device_class': None, 'unit': 'V'},
    {'name': 'INPA4', 'key': 'inpa4', 'device_class': None, 'unit': 'V'},
    {'name': 'INPA5', 'key': 'inpa5', 'device_class': None, 'unit': 'V'},
    {'name': 'INPA6', 'key': 'inpa6', 'device_class': None, 'unit': 'V'},
    {'name': 'POWER1', 'key': 'power1', 'device_class': DEVICE_CLASS_POWER, 'unit': POWER_WATT},
    {'name': 'POWER2', 'key': 'power2', 'device_class': DEVICE_CLASS_POWER, 'unit': POWER_WATT},
    {'name': 'POWER3', 'key': 'power3', 'device_class': DEVICE_CLASS_POWER, 'unit': POWER_WATT},
    {'name': 'POWER4', 'key': 'power4', 'device_class': DEVICE_CLASS_POWER, 'unit': POWER_WATT},
    {'name': 'ENERGY1', 'key': 'energy1', 'device_class': None, 'unit': ENERGY_KILO_WATT_HOUR},
    {'name': 'ENERGY2', 'key': 'energy2', 'device_class': None, 'unit': ENERGY_KILO_WATT_HOUR},
    {'name': 'ENERGY3', 'key': 'energy3', 'device_class': None, 'unit': ENERGY_KILO_WATT_HOUR},
    {'name': 'ENERGY4', 'key': 'energy4', 'device_class': None, 'unit': ENERGY_KILO_WATT_HOUR},
    {'name': 'Duralux Temperature', 'key': 'duralux0', 'device_class': None, 'unit': TEMP_CELSIUS},
    {'name': 'Duralux PvVolt', 'key': 'duralux1', 'device_class': None, 'unit': 'V'},
    {'name': 'Duralux GridCurrent', 'key': 'duralux2', 'device_class': None, 'unit': 'A'},
    {'name': 'Duralux GridVolt', 'key': 'duralux3', 'device_class': None, 'unit': 'V'},
    {'name': 'Duralux GridFreq', 'key': 'duralux4', 'device_class': None, 'unit': 'Hz'},
    {'name': 'Duralux OutputWattage', 'key': 'duralux5', 'device_class': None, 'unit': POWER_WATT},
    {'name': 'Duralux GridZero', 'key': 'duralux6', 'device_class': None, 'unit': 'V'},
    {'name': 'Duralux Energy', 'key': 'duralux7', 'device_class': None, 'unit': 'Wh'},
    {'name': 'Duralux TotalWorkHour', 'key': 'duralux8', 'device_class': None, 'unit': 'h'},
    {'name': 'SDM1', 'key': 'sdm1', 'device_class': None, 'unit': None},
    {'name': 'SDM2', 'key': 'sdm2', 'device_class': None, 'unit': None},
    {'name': 'SDM3', 'key': 'sdm3', 'device_class': None, 'unit': None},
    {'name': 'SDM4', 'key': 'sdm4', 'device_class': None, 'unit': None},
    {'name': 'SDM5', 'key': 'sdm5', 'device_class': None, 'unit': None},
    {'name': 'SDM6', 'key': 'sdm6', 'device_class': None, 'unit': None},
    {'name': 'SDM7', 'key': 'sdm7', 'device_class': None, 'unit': None},
    {'name': 'SDM8', 'key': 'sdm8', 'device_class': None, 'unit': None},
    {'name': 'SDM9', 'key': 'sdm9', 'device_class': None, 'unit': None},
    {'name': 'SDM10', 'key': 'sdm10', 'device_class': None, 'unit': None},
    {'name': 'SDM11', 'key': 'sdm11', 'device_class': None, 'unit': None},
    {'name': 'SDM12', 'key': 'sdm12', 'device_class': None, 'unit': None},
    {'name': 'SDM13', 'key': 'sdm13', 'device_class': None, 'unit': None},
    {'name': 'SDM14', 'key': 'sdm14', 'device_class': None, 'unit': None},
    {'name': 'SDM15', 'key': 'sdm15', 'device_class': None, 'unit': None},
    {'name': 'SDM16', 'key': 'sdm16', 'device_class': None, 'unit': None},
    {'name': 'SDM17', 'key': 'sdm17', 'device_class': None, 'unit': None},
    {'name': 'SDM18', 'key': 'sdm18', 'device_class': None, 'unit': None},
    {'name': 'SDM19', 'key': 'sdm19', 'device_class': None, 'unit': None},
    {'name': 'SDM20', 'key': 'sdm20', 'device_class': None, 'unit': None},
    {'name': 'SDM21', 'key': 'sdm21', 'device_class': None, 'unit': None},
    {'name': 'SDM22', 'key': 'sdm22', 'device_class': None, 'unit': None},
    {'name': 'SDM23', 'key': 'sdm23', 'device_class': None, 'unit': None},
    {'name': 'SDM24', 'key': 'sdm24', 'device_class': None, 'unit': None},
    {'name': 'SDM25', 'key': 'sdm25', 'device_class': None, 'unit': None},
    {'name': 'SDM26', 'key': 'sdm26', 'device_class': None, 'unit': None},
    {'name': 'SDM27', 'key': 'sdm27', 'device_class': None, 'unit': None},
    {'name': 'SDM28', 'key': 'sdm28', 'device_class': None, 'unit': None},
    {'name': 'SDM29', 'key': 'sdm29', 'device_class': None, 'unit': None},
]


async def async_setup_entry(
        hass: HomeAssistantType,
        entry: ConfigEntry,
        async_add_enities: Callable[[List[Entity], bool], None],
) -> None:
    """Set up LK3 sensor based on a config entry."""
    reader: LK3Reader = hass.data[DOMAIN][entry.entry_id][DATA_LK3_CLIENT]

    sensors = [
        LK3Sensor(
            entry.entry_id,
            reader,
            item['name'],
            None,
            item['unit'],
            item['key'],
            item['device_class']
        ) for item in SENSOR_LIST
    ]
    async_add_enities(sensors, True)


class LK3Sensor(LK3DeviceEntity):
    """LK3Sensor."""

    def __init__(
            self,
            entry_id: str,
            reader: LK3Reader,
            name: str,
            icon: str,
            unit_of_measurement: str,
            key: str,
            device_class: str,
    ) -> None:
        """Initialize LK3 sensor."""
        self._state = None
        self._unit_of_measurement = unit_of_measurement
        self._key = key
        self._device_class = device_class
        super().__init__(entry_id, reader, name, icon)

    @property
    def unique_id(self) -> str:
        """Return the unique ID for this sensor."""
        return f"{self.reader.device.info.mac_address}_{self._key}"

    @property
    def device_class(self) -> Optional[str]:
        """Return the class of this sensor."""
        return self._device_class

    @property
    def state(self) -> Union[None, str, int, float]:
        """Return the state of the sensor."""
        return self._state

    @property
    def unit_of_measurement(self) -> str:
        """Return the unit this state is expressed in."""
        return self._unit_of_measurement

    @property
    def entity_registry_enabled_default(self) -> bool:
        """Return the default state of registered entity."""
        return False

    async def _lk3_update(self) -> None:
        """Update LK3 entity."""
        self._state = self.reader.device.state[self._key]
